@@ -141,7 +141,7 @@ static int link_is_dir (const char *folder, const char *path)
 }
 
 static const char *
-folder_format_str (char *dest, size_t destlen, char op, const char *src,
+folder_format_str (char *dest, size_t destlen, size_t col, char op, const char *src,
 		   const char *fmt, const char *ifstring, const char *elsestring,
 		   unsigned long data, format_flag flags)
 {
@@ -265,18 +265,19 @@ folder_format_str (char *dest, size_t destlen, char op, const char *src,
     case 's':
       if (folder->ff->st != NULL)
       {
-	snprintf (tmp, sizeof (tmp), "%%%sld", fmt);
-	snprintf (dest, destlen, tmp, (long) folder->ff->st->st_size);
+	mutt_pretty_size(fn, sizeof(fn), folder->ff->st->st_size);
+	snprintf (tmp, sizeof (tmp), "%%%ss", fmt);
+	snprintf (dest, destlen, tmp, fn);
       }
       else
 	mutt_format_s (dest, destlen, fmt, "");
       break;
-      
+
     case 't':
       snprintf (tmp, sizeof (tmp), "%%%sc", fmt);
       snprintf (dest, destlen, tmp, folder->ff->tagged ? '*' : ' ');
       break;
-      
+
     case 'u':
       if (folder->ff->st != NULL)
       {
@@ -291,7 +292,7 @@ folder_format_str (char *dest, size_t destlen, char op, const char *src,
       else
 	mutt_format_s (dest, destlen, fmt, "");
       break;
-      
+
     default:
       snprintf (tmp, sizeof (tmp), "%%%sc", fmt);
       snprintf (dest, destlen, tmp, op);
@@ -299,9 +300,9 @@ folder_format_str (char *dest, size_t destlen, char op, const char *src,
   }
 
   if (optional)
-    mutt_FormatString (dest, destlen, ifstring, folder_format_str, data, 0);
+    mutt_FormatString (dest, destlen, col, ifstring, folder_format_str, data, 0);
   else if (flags & M_FORMAT_OPTIONAL)
-    mutt_FormatString (dest, destlen, elsestring, folder_format_str, data, 0);
+    mutt_FormatString (dest, destlen, col, elsestring, folder_format_str, data, 0);
 
   return (src);
 }
@@ -482,7 +483,7 @@ static void folder_entry (char *s, size_t slen, MUTTMENU *menu, int num)
   folder.ff = &((struct folder_file *) menu->data)[num];
   folder.num = num;
   
-  mutt_FormatString (s, slen, NONULL(FolderFormat), folder_format_str, 
+  mutt_FormatString (s, slen, 0, NONULL(FolderFormat), folder_format_str, 
       (unsigned long) &folder, M_FORMAT_ARROWCURSOR);
 }
 
@@ -619,7 +620,15 @@ void _mutt_select_file (char *f, size_t flen, int flags, char ***files, int *num
       imap_browse (LastDir, &state);
       browser_sort (&state);
     }
+    else
 #endif
+    {
+      i = mutt_strlen (LastDir);
+      while (i && LastDir[--i] == '/')
+        LastDir[i] = '\0';
+      if (!LastDir[0])
+        getcwd (LastDir, sizeof (LastDir));
+    }
   }
 
   *f = 0;
