@@ -248,8 +248,8 @@ void imap_expunge_mailbox (IMAP_DATA* idata)
   HEADER* h;
   int i, cacheno;
 
-#if USE_HCACHE
-  char uidbuf[32];
+#ifdef USE_HCACHE
+  idata->hcache = imap_hcache_open (idata, NULL);
 #endif
 
   for (i = 0; i < idata->ctx->msgcount; i++)
@@ -265,11 +265,7 @@ void imap_expunge_mailbox (IMAP_DATA* idata)
 
       imap_cache_del (idata, h);
 #if USE_HCACHE
-      if (idata->hcache)
-      {
-        sprintf (uidbuf, "/%u", HEADER_DATA(h)->uid);
-        mutt_hcache_delete (idata->hcache, uidbuf, imap_hcache_keylen);
-      }
+      imap_hcache_del (idata, HEADER_DATA(h)->uid);
 #endif
 
       /* free cached body from disk, if necessary */
@@ -285,6 +281,10 @@ void imap_expunge_mailbox (IMAP_DATA* idata)
     }
   }
 
+#if USE_HCACHE
+  imap_hcache_close (idata);
+#endif
+
   /* We may be called on to expunge at any time. We can't rely on the caller
    * to always know to rethread */
   mx_update_tables (idata->ctx, 0);
@@ -742,9 +742,7 @@ int imap_open_mailbox (CONTEXT* ctx)
   ctx->hdrs = safe_calloc (count, sizeof (HEADER *));
   ctx->v2r = safe_calloc (count, sizeof (int));
   ctx->msgcount = 0;
-#ifdef USE_HCACHE
-  idata->hcache = imap_hcache_open (idata, idata->ctx->path);
-#endif
+
   if (count && (imap_read_headers (idata, 0, count-1) < 0))
   {
     mutt_error _("Error opening mailbox");
@@ -757,9 +755,6 @@ int imap_open_mailbox (CONTEXT* ctx)
   return 0;
 
  fail:
-#ifdef USE_HCACHE
-  mutt_hcache_close (idata->hcache);
-#endif
   if (idata->state == IMAP_SELECTED)
     idata->state = IMAP_AUTHENTICATED;
  fail_noidata:
@@ -1106,9 +1101,6 @@ int imap_sync_mailbox (CONTEXT* ctx, int expunge, int* index_hint)
   int deleted;
   int n;
   int rc;
-#if USE_HCACHE
-  char uidbuf[32];
-#endif
   
   idata = (IMAP_DATA*) ctx->data;
 
@@ -1153,20 +1145,23 @@ int imap_sync_mailbox (CONTEXT* ctx, int expunge, int* index_hint)
     }
   }
 
+#if USE_HCACHE
+  idata->hcache = imap_hcache_open (idata, NULL);
+#endif
+
   /* save messages with real (non-flag) changes */
   for (n = 0; n < ctx->msgcount; n++)
   {
     h = ctx->hdrs[n];
 
     if (h->deleted)
+    {
       imap_cache_del (idata, h);
 #if USE_HCACHE
-    if (idata->hcache && h->deleted)
-    {
-      sprintf (uidbuf, "/%u", HEADER_DATA(h)->uid);
-      mutt_hcache_delete (idata->hcache, uidbuf, imap_hcache_keylen);
-    }
+      imap_hcache_del (idata, HEADER_DATA(h)->uid);
 #endif
+    }
+    
     if (h->active && h->changed)
     {
       /* if the message has been rethreaded or attachments have been deleted
@@ -1189,6 +1184,10 @@ int imap_sync_mailbox (CONTEXT* ctx, int expunge, int* index_hint)
     }
   }
 
+#if USE_HCACHE
+  imap_hcache_close (idata);
+#endif
+
   /* sync +/- flags for the five flags mutt cares about */
   rc = 0;
 
@@ -1262,7 +1261,11 @@ int imap_sync_mailbox (CONTEXT* ctx, int expunge, int* index_hint)
     idata->state = IMAP_AUTHENTICATED;
   }
 
+  if (option (OPTMESSAGECACHECLEAN))
+    imap_cache_clean (idata);
+
   rc = 0;
+
  out:
   if (cmd.data)
     FREE (&cmd.data);
@@ -1322,9 +1325,6 @@ int imap_close_mailbox (CONTEXT* ctx)
     }
   }
 
-#ifdef USE_HCACHE
-  mutt_hcache_close (idata->hcache);
-#endif
   mutt_bcache_close (&idata->bcache);
 
   return 0;
@@ -1571,8 +1571,6 @@ IMAP_STATUS* imap_mboxcache_get (IMAP_DATA* idata, const char* mbox, int create)
   IMAP_STATUS scache;
 #ifdef USE_HCACHE
   header_cache_t *hc = NULL;
-  ciss_url_t url;
-  char urlstr[LONG_STRING];
   unsigned int *uidvalidity = NULL;
   unsigned int *uidnext = NULL;
 #endif
@@ -1598,10 +1596,7 @@ IMAP_STATUS* imap_mboxcache_get (IMAP_DATA* idata, const char* mbox, int create)
   }
 
 #ifdef USE_HCACHE
-  mutt_account_tourl (&idata->conn->account, &url);
-  url.path = (char*)mbox;
-  url_ciss_tostring (&url, urlstr, sizeof (urlstr), 0);
-  hc = mutt_hcache_open (HeaderCache, urlstr);
+  hc = imap_hcache_open (idata, mbox);
   if (hc)
   {
     uidvalidity = mutt_hcache_fetch_raw (hc, "/UIDVALIDITY", imap_hcache_keylen);
