@@ -119,6 +119,8 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
   idata->newMailCount = 0;
 
 #if USE_HCACHE
+  idata->hcache = imap_hcache_open (idata, NULL);
+
   if (idata->hcache && !msgbegin)
   {
     uid_validity = mutt_hcache_fetch_raw (idata->hcache, "/UIDVALIDITY", imap_hcache_keylen);
@@ -154,15 +156,20 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
         rc = imap_cmd_step (idata);
         if (rc != IMAP_CMD_CONTINUE)
 	{
-	  imap_free_header_data ((void**) &h.data);
+          /* suppress GCC aliasing warning */
+	  imap_free_header_data ((void**) (void*) &h.data);
           break;
 	}
 
+        /* hole in the header cache */
+        if (!evalhc)
+          continue;
+
         if ((mfhrc = msg_fetch_header (ctx, &h, idata->buf, NULL)) == -1)
           continue;
         else if (mfhrc < 0)
 	{
-	  imap_free_header_data ((void**) &h.data);
+	  imap_free_header_data ((void**) (void*) &h.data);
           break;
 	}
 
@@ -187,9 +194,12 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
           ctx->size += ctx->hdrs[idx]->content->length;
         }
 	else
-	  /* bad header in the cache, we'll have to refetch.
-	   * TODO: consider the possibility of a holey cache. */
-          imap_free_header_data((void**) &h.data);
+        {
+	  /* bad header in the cache, we'll have to refetch. */
+          dprint (3, (debugfile, "bad cache entry at %d, giving up\n", h.sid - 1));
+          imap_free_header_data((void**) (void*) &h.data);
+          evalhc = 0;
+        }
       }
       while (rc != IMAP_CMD_OK && mfhrc == -1);
       if (rc == IMAP_CMD_OK)
@@ -197,7 +207,8 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
       if ((mfhrc < -1) || ((rc != IMAP_CMD_CONTINUE) && (rc != IMAP_CMD_OK)))
       {
         if (h.data)
-          imap_free_header_data ((void**) &h.data);
+          imap_free_header_data ((void**) (void*) &h.data);
+        imap_hcache_close (idata);
         fclose (fp);
         return -1;
       }
@@ -214,19 +225,11 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
   {
     mutt_progress_update (&progress, msgno + 1);
 
+    /* we may get notification of new mail while fetching headers */
     if (msgno + 1 > fetchlast)
     {
-      fetchlast = msgno + 1;
-      while((fetchlast <= msgend) && (! ctx->hdrs[fetchlast]))
-        fetchlast++;
+      fetchlast = msgend + 1;
 
-      /*
-       * Make one request for everything. This makes fetching headers an
-       * order of magnitude faster if you have a large mailbox.
-       *
-       * If we get more messages while doing this, we make another
-       * request for all the new messages.
-       */
       snprintf (buf, sizeof (buf),
         "FETCH %d:%d (UID FLAGS INTERNALDATE RFC822.SIZE %s)", msgno + 1,
         fetchlast, hdrreq);
@@ -298,7 +301,10 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
     if ((mfhrc < -1) || ((rc != IMAP_CMD_CONTINUE) && (rc != IMAP_CMD_OK)))
     {
       if (h.data)
-        imap_free_header_data ((void**) &h.data);
+        imap_free_header_data ((void**) (void*) &h.data);
+#if USE_HCACHE
+      imap_hcache_close (idata);
+#endif
       fclose (fp);
       return -1;
     }
@@ -328,6 +334,8 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
   if (idata->uidnext > 1)
     mutt_hcache_store_raw (idata->hcache, "/UIDNEXT", &idata->uidnext,
 			   sizeof (idata->uidnext), imap_hcache_keylen);
+
+  imap_hcache_close (idata);
 #endif /* USE_HCACHE */
 
   fclose(fp);
@@ -838,35 +846,12 @@ int imap_copy_messages (CONTEXT* ctx, HEADER* h, char* dest, int delete)
 
 static body_cache_t *msg_cache_open (IMAP_DATA *idata)
 {
-  char *s;
-  char *p = idata->mailbox;
   char mailbox[_POSIX_PATH_MAX];
-  size_t mlen = sizeof (mailbox);
 
   if (idata->bcache)
     return idata->bcache;
 
-  mailbox[0] = '\0';
-
-  for (s = mailbox; p && *p && mlen; mlen--)
-  {
-    if (*p == idata->delim)
-    {
-      *s = '/';
-      /* simple way to avoid collisions with UIDs */
-      if (*(p + 1) >= '0' && *(p + 1) <= '9')
-      {
-	mlen--;
-	if (mlen)
-	  *++s = '_';
-      }
-    }
-    else
-      *s = *p;
-    p++;
-    s++;
-  }
-  *s = '\0';
+  imap_cachepath (idata, idata->mailbox, mailbox, sizeof (mailbox));
 
   return mutt_bcache_open (&idata->conn->account, mailbox);
 }
@@ -920,6 +905,37 @@ int imap_cache_del (IMAP_DATA* idata, HEADER* h)
   return mutt_bcache_del (idata->bcache, id);
 }
 
+static int msg_cache_clean_cb (const char* id, body_cache_t* bcache, void* data)
+{
+  unsigned int uv, uid, n;
+  IMAP_DATA* idata = (IMAP_DATA*)data;
+
+  if (sscanf (id, "%u-%u", &uv, &uid) != 2)
+    return 0;
+
+  /* bad UID */
+  if (uv != idata->uid_validity)
+    mutt_bcache_del (bcache, id);
+
+  /* TODO: presort UIDs, walk in order */
+  for (n = 0; n < idata->ctx->msgcount; n++)
+  {
+    if (uid == HEADER_DATA(idata->ctx->hdrs[n])->uid)
+      return 0;
+  }
+  mutt_bcache_del (bcache, id);
+
+  return 0;
+}
+
+int imap_cache_clean (IMAP_DATA* idata)
+{
+  idata->bcache = msg_cache_open (idata);
+  mutt_bcache_list (idata->bcache, msg_cache_clean_cb, idata);
+
+  return 0;
+}
+
 /* imap_add_keywords: concatenate custom IMAP tags to list, if they
  *   appear in the folder flags list. Why wouldn't they? */
 void imap_add_keywords (char* s, HEADER* h, LIST* mailbox_flags, size_t slen)
