@@ -71,21 +71,43 @@ int imap_expand_path (char* path, size_t len)
 }
 
 #ifdef USE_HCACHE
+static int imap_hcache_namer (const char* path, char* dest, size_t dlen)
+{
+  return snprintf (dest, dlen, "%s.hcache", path);
+}
+
 header_cache_t* imap_hcache_open (IMAP_DATA* idata, const char* path)
 {
   IMAP_MBOX mx;
   ciss_url_t url;
   char cachepath[LONG_STRING];
+  char mbox[LONG_STRING];
 
-  if (imap_parse_path (path, &mx) < 0)
-    return NULL;
+  if (path)
+    imap_cachepath (idata, path, mbox, sizeof (mbox));
+  else
+  {
+    if (!idata->ctx || imap_parse_path (idata->ctx->path, &mx) < 0)
+      return NULL;
+
+    imap_cachepath (idata, mx.mbox, mbox, sizeof (mbox));
+    FREE (&mx.mbox);
+  }
 
   mutt_account_tourl (&idata->conn->account, &url);
-  url.path = mx.mbox;
-  url_ciss_tostring (&url, cachepath, sizeof (cachepath), 0);
-  FREE (&mx.mbox);
+  url.path = mbox;
+  url_ciss_tostring (&url, cachepath, sizeof (cachepath), U_PATH);
 
-  return mutt_hcache_open (HeaderCache, cachepath);
+  return mutt_hcache_open (HeaderCache, cachepath, imap_hcache_namer);
+}
+
+void imap_hcache_close (IMAP_DATA* idata)
+{
+  if (!idata->hcache)
+    return;
+
+  mutt_hcache_close (idata->hcache);
+  idata->hcache = NULL;
 }
 
 HEADER* imap_hcache_get (IMAP_DATA* idata, unsigned int uid)
@@ -94,7 +116,10 @@ HEADER* imap_hcache_get (IMAP_DATA* idata, unsigned int uid)
   unsigned int* uv;
   HEADER* h = NULL;
 
-  sprintf(key, "/%u", uid);
+  if (!idata->hcache)
+    return NULL;
+
+  sprintf (key, "/%u", uid);
   uv = (unsigned int*)mutt_hcache_fetch (idata->hcache, key,
                                          imap_hcache_keylen);
   if (uv)
@@ -111,10 +136,24 @@ int imap_hcache_put (IMAP_DATA* idata, HEADER* h)
 {
   char key[16];
 
-  sprintf(key, "/%u", HEADER_DATA (h)->uid);
+  if (!idata->hcache)
+    return -1;
+
+  sprintf (key, "/%u", HEADER_DATA (h)->uid);
   return mutt_hcache_store (idata->hcache, key, h, idata->uid_validity,
                             imap_hcache_keylen);
 }
+
+int imap_hcache_del (IMAP_DATA* idata, unsigned int uid)
+{
+  char key[16];
+
+  if (!idata->hcache)
+    return -1;
+
+  sprintf (key, "/%u", uid);
+  return mutt_hcache_delete (idata->hcache, key, imap_hcache_keylen);
+}
 #endif
 
 /* imap_parse_path: given an IMAP mailbox name, return host, port
@@ -370,6 +409,32 @@ char *imap_fix_path (IMAP_DATA *idata, char *mailbox, char *path,
   return path;
 }
 
+void imap_cachepath(IMAP_DATA* idata, const char* mailbox, char* dest,
+                    size_t dlen)
+{
+  char* s;
+  const char* p = mailbox;
+
+  for (s = dest; p && *p && dlen; dlen--)
+  {
+    if (*p == idata->delim)
+    {
+      *s = '/';
+      /* simple way to avoid collisions with UIDs */
+      if (*(p + 1) >= '0' && *(p + 1) <= '9')
+      {
+	if (--dlen)
+	  *++s = '_';
+      }
+    }
+    else
+      *s = *p;
+    p++;
+    s++;
+  }
+  *s = '\0';
+}
+
 /* imap_get_literal_count: write number of bytes in an IMAP literal into
  *   bytes, return 0 on success, -1 on failure. */
 int imap_get_literal_count(const char *buf, long *bytes)
